@@ -17,7 +17,7 @@
 #   
 #   LICENCE
 #   
-#   FMRIB Software Library, Release 4.0 (c) 2007, The University of
+#   FMRIB Software Library, Release 5.0 (c) 2012, The University of
 #   Oxford (the "Software")
 #   
 #   The Software remains the property of the University of Oxford ("the
@@ -66,7 +66,7 @@
 #   interested in using the Software commercially, please contact Isis
 #   Innovation Limited ("Isis"), the technology transfer company of the
 #   University, to negotiate a licence. Contact details are:
-#   innovation@isis.ox.ac.uk quoting reference DE/1112.
+#   innovation@isis.ox.ac.uk quoting reference DE/9564.
 
 #{{{ setup
 
@@ -257,6 +257,8 @@ $w.nb raise search
 	    -variable reg($w,interp) -value trilinear -anchor w -command "flirt:updateinterp $w $interplf"
     radiobutton $w.nearestneighbour -text "Nearest Neighbour" \
 	    -variable reg($w,interp) -value nearestneighbour -anchor w -command "flirt:updateinterp $w $interplf"
+    radiobutton $w.spline -text "Spline" \
+	    -variable reg($w,interp) -value spline -anchor w -command "flirt:updateinterp $w $interplf"
     radiobutton $w.sinc -text "Sinc" \
 	    -variable reg($w,interp) -value sinc -anchor w -command "flirt:updateinterp $w $interplf"
     set reg($w,sincwidth) 7
@@ -273,7 +275,7 @@ $w.nb raise search
     
     # ---- pack ----
     pack $w.interpbanner $w.trilinear -in $interplf -side top -anchor w -padx 3
-    pack $w.nearestneighbour $w.sinc -in $interplf -side top -anchor w -padx 3
+    pack $w.nearestneighbour $w.spline $w.sinc -in $interplf -side top -anchor w -padx 3
     set reg($w,interp) trilinear
 
     pack $w.swinbanner -in $w.swinopt -side top -anchor w -padx 3
@@ -325,7 +327,7 @@ pack $w.wgt $w.iwgt -in $weightlf -side top -anchor w -padx 3 -pady $PADY
 	[winfo toplevel %W].cancel invoke
     }
 
-    button $w.help -command "FmribWebHelp file: ${FSLDIR}/doc/flirt/index.html" \
+    button $w.help -command "FmribWebHelp file: ${FSLDIR}/doc/redirects/flirt.html" \
 	    -text "Help" -width 5
 
     #{{{ Utils
@@ -492,6 +494,7 @@ proc flirt:updateinterp { w interplf } {
 	pack forget $w.swinopt
 	pack forget $w.sincwidth
     }
+    $w.nb compute_size
 }
 
 #}}}
