@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 #  combobox.tcl
 #  This file is part of Unifix BWidget Toolkit
-#  $Id: combobox.tcl,v 1.1 2006/11/28 15:34:01 mwebster Exp $
+#  $Id: combobox.tcl,v 1.2 2008/01/08 16:31:24 mwebster Exp $
 # ----------------------------------------------------------------------------
 #  Index of commands:
 #     - ComboBox::create
@@ -9,6 +9,7 @@
 #     - ComboBox::cget
 #     - ComboBox::setvalue
 #     - ComboBox::getvalue
+#     - ComboBox::clearvalue
 #     - ComboBox::_create_popup
 #     - ComboBox::_mapliste
 #     - ComboBox::_unmapliste
@@ -39,6 +40,7 @@ namespace eval ComboBox {
 	{-postcommand  String	  ""   0}
 	{-expand       Enum	  none 0 {none tab}}
 	{-autocomplete Boolean	  0    0}
+        {-autopost     Boolean    0    0}
         {-bwlistbox    Boolean    0    0}
         {-listboxwidth Int        0    0}
         {-hottrack     Boolean    0    0}
@@ -92,6 +94,13 @@ proc ComboBox::create { path args } {
 	::bind $path.e <KeyRelease> [list $path _auto_complete %K]
     }
 
+    if {[Widget::cget $path -autopost]} {
+        ::bind $path.e <KeyRelease> +[list $path _auto_post %K]
+    } else {
+        ::bind $entry <Key-Up>	  [list ComboBox::_unmapliste $path]
+        ::bind $entry <Key-Down>  [list ComboBox::_mapliste $path]
+    }
+
     if {[string equal $::tcl_platform(platform) "unix"]} {
 	set ipadx 0
 	set width 11
@@ -101,12 +110,11 @@ proc ComboBox::create { path args } {
     }
     set height [winfo reqheight $entry]
     set arrow [eval [list ArrowButton::create $path.a] $maps(.a) \
-		   -width $width -height $height \
-		   -highlightthickness 0 -borderwidth 1 -takefocus 0 \
-		   -dir	  bottom \
-		   -type  button \
-		   -ipadx $ipadx \
-		   -command [list [list ComboBox::_mapliste $path]]]
+		   [list -width $width -height $height \
+			-highlightthickness 0 -borderwidth 1 -takefocus 0 \
+			-dir bottom -type  button -ipadx $ipadx \
+			-command [list ComboBox::_mapliste $path] \
+		       ]]
 
     pack $arrow -side right -fill y
     pack $entry -side left  -fill both -expand yes
@@ -123,8 +131,6 @@ proc ComboBox::create { path args } {
     }
 
     ::bind $path  <ButtonPress-1> [list ComboBox::_unmapliste $path]
-    ::bind $entry <Key-Up>	  [list ComboBox::_unmapliste $path]
-    ::bind $entry <Key-Down>	  [list ComboBox::_mapliste $path]
     ::bind $entry <Control-Up>	  [list ComboBox::_modify_value $path previous]
     ::bind $entry <Control-Down>  [list ComboBox::_modify_value $path next]
     ::bind $entry <Control-Prior> [list ComboBox::_modify_value $path first]
@@ -176,7 +182,7 @@ proc ComboBox::configure { path args } {
 
 
     set list [list -images -values -bwlistbox -hottrack]
-    foreach {ci cv cb ch} [eval Widget::hasChangedX $path $list] { break }
+    foreach {ci cv cb ch} [eval [linsert $list 0 Widget::hasChangedX $path]] { break }
 
     if { $ci } {
         set images [Widget::cget $path -images]
@@ -353,7 +359,7 @@ proc ComboBox::insert { path idx args } {
 
     if {[Widget::cget $path -bwlistbox]} {
         set l [$path getlistbox]
-        set i [eval $l insert $idx #auto $args]
+        set i [eval [linsert $args 0 $l insert $idx #auto]]
         set text [$l itemcget $i -text]
         if {$idx == "end"} {
             lappend values $text
@@ -361,10 +367,17 @@ proc ComboBox::insert { path idx args } {
             set values [linsert $values $idx $text]
         }
     } else {
-        set values [eval linsert [list $values] $idx $args]
+        set values [eval [list linsert $values $idx] $args]
     }
 }
 
+# ----------------------------------------------------------------------------
+#  Command ComboBox::clearvalue
+# ----------------------------------------------------------------------------
+proc ComboBox::clearvalue { path } {
+    Entry::configure $path.e -text ""
+}
+
 # ----------------------------------------------------------------------------
 #  Command ComboBox::_create_popup
 # ----------------------------------------------------------------------------
@@ -396,14 +409,13 @@ proc ComboBox::_create_popup { path } {
 
     toplevel            $shell -relief solid -bd 1
     wm withdraw         $shell
-    update idletasks
+    update idle
     wm overrideredirect $shell 1
     wm transient        $shell [winfo toplevel $path]
-    wm withdraw         $shell
     catch { wm attributes $shell -topmost 1 }
 
     set sw [ScrolledWindow $shell.sw -managed 0 -size $sbwidth -ipad 0]
-    
+
     if {$bw} {
         set listb  [ListBox $shell.listb \
                 -relief flat -borderwidth 0 -highlightthickness 0 \
@@ -419,8 +431,8 @@ proc ComboBox::_create_popup { path } {
         foreach value $values image $images {
             $listb insert end #auto -text $value -image $image
         }
-	$listb bindText  <1> "ComboBox::_select $path"
-	$listb bindImage <1> "ComboBox::_select $path"
+	$listb bindText  <1> [list ComboBox::_select $path]
+	$listb bindImage <1> [list ComboBox::_select $path]
         if {[Widget::cget $path -hottrack]} {
             $listb bindText  <Enter> [list $listb selection set]
             $listb bindImage <Enter> [list $listb selection set]
@@ -445,7 +457,7 @@ proc ComboBox::_create_popup { path } {
     pack $sw -fill both -expand yes
     $sw setwidget $listb
 
-    ::bind $listb <Return>   "ComboBox::_select $path \[%W curselection]"
+    ::bind $listb <Return> "ComboBox::_select [list $path] \[%W curselection\]"
     ::bind $listb <Escape>   [list ComboBox::_unmapliste $path]
     ::bind $listb <FocusOut> [list ComboBox::_focus_out $path]
 }
@@ -518,7 +530,7 @@ proc ComboBox::_mapliste { path } {
     if { [Widget::cget $path -state] == "disabled" } {
         return
     }
-    if { [set cmd [Widget::getMegawidgetOption $path -postcommand]] != "" } {
+    if {[llength [set cmd [Widget::getMegawidgetOption $path -postcommand]]]} {
         uplevel \#0 $cmd
     }
     if { ![llength [Widget::getMegawidgetOption $path -values]] } {
@@ -593,7 +605,7 @@ proc ComboBox::_select { path index } {
     if { $index != -1 } {
         if { [setvalue $path @$index] } {
 	    set cmd [Widget::getMegawidgetOption $path -modifycmd]
-            if { $cmd != "" } {
+            if {[llength $cmd]} {
                 uplevel \#0 $cmd
             }
         }
@@ -607,10 +619,9 @@ proc ComboBox::_select { path index } {
 #  Command ComboBox::_modify_value
 # ----------------------------------------------------------------------------
 proc ComboBox::_modify_value { path direction } {
-    if { [setvalue $path $direction] } {
-        if { [set cmd [Widget::getMegawidgetOption $path -modifycmd]] != "" } {
-            uplevel \#0 $cmd
-        }
+    if {[setvalue $path $direction]
+        && [llength [set cmd [Widget::getMegawidgetOption $path -modifycmd]]]} {
+	uplevel \#0 $cmd
     }
 }
 
@@ -711,17 +722,16 @@ proc ComboBox::_traverse_in { path } {
 #  Command ComboBox::_focus_out
 # ----------------------------------------------------------------------------
 proc ComboBox::_focus_out { path } {
-    if {[focus] == ""} {
-	# we lost focus to some other app, make sure we drop the listbox
+    if {[string first $path [focus]] != 0} {
+	# we lost focus to some other app or window, so remove the listbox
 	return [_unmapliste $path 0]
     }
 }
 
 proc ComboBox::_auto_complete { path key } {
-    ## Anything that is all lowercase is either a letter, number
-    ## or special key we're ok with.  Everything else is a
-    ## functional key of some kind.
-    if {[string tolower $key] != $key} { return }
+    ## Any key string with more than one character and is not entirely
+    ## lower-case is considered a function key and is thus ignored.
+    if {[string length $key] > 1 && [string tolower $key] != $key} { return }
 
     set text [string map [list {[} {\[} {]} {\]}] [$path.e get]]
     if {[string equal $text ""]} { return }
@@ -734,3 +744,66 @@ proc ComboBox::_auto_complete { path key } {
     $path.e icursor $idx
     $path.e select range insert end
 }
+
+proc ComboBox::_auto_post { path key } {
+    if {[string equal $key "Escape"] || [string equal $key "Return"]} {
+        _unmapliste $path
+        return
+    }
+    if {[catch {$path.shell.listb curselection} x] || $x == ""} {
+        if {[string equal $key "Up"]} {
+            _unmapliste $path
+            return
+        }
+        set x -1
+    }
+    if {([string length $key] > 1 && [string tolower $key] != $key) && \
+            [string equal $key "Backspace"] != 0 && \
+            [string equal $key "Up"] != 0 && \
+            [string equal $key "Down"] != 0} {
+        return
+    }
+
+    # post the listbox
+    _create_popup $path
+    set width [Widget::cget $path -listboxwidth]
+    if {!$width} { set width [winfo width $path] }
+    BWidget::place $path.shell $width 0 below $path
+    wm deiconify $path.shell
+    BWidget::grab release $path
+    BWidget::focus release $path.shell.listb 1
+    focus -force $path.e
+
+    set values [Widget::cget $path -values]
+    switch -- $key {
+        Up {
+            if {[incr x -1] < 0} {
+                set x 0
+            } else {
+                Entry::configure $path.e -text [lindex $values $x]
+            }
+        }
+        Down {
+            if {[incr x] >= [llength $values]} {
+                set x [expr {[llength $values] - 1}]
+            } else {
+                Entry::configure $path.e -text [lindex $values $x]
+            }
+        }
+        default {
+            # auto-select within the listbox the item closest to the entry's value
+            set text [string map [list {[} {\[} {]} {\]}] [$path.e get]]
+            if {[string equal $text ""]} {
+                set x 0
+            } else {
+                set x [lsearch $values $text*]
+            }
+        }
+    }
+
+    if {$x >= 0} {
+        $path.shell.listb selection clear 0 end
+        $path.shell.listb selection set $x
+        $path.shell.listb see $x
+    }
+}
