@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 #  tree.tcl
 #  This file is part of Unifix BWidget Toolkit
-#  $Id: tree.tcl,v 1.1 2006/11/28 15:34:01 mwebster Exp $
+#  $Id: tree.tcl,v 1.2 2008/01/08 16:31:25 mwebster Exp $
 # ----------------------------------------------------------------------------
 #  Index of commands:
 #     - Tree::create
@@ -10,6 +10,7 @@
 #     - Tree::insert
 #     - Tree::itemconfigure
 #     - Tree::itemcget
+#     - Tree::bindArea
 #     - Tree::bindText
 #     - Tree::bindImage
 #     - Tree::delete
@@ -60,7 +61,7 @@ namespace eval Tree {
             {-data       String     ""      0}
             {-open       Boolean    0       0}
 	    {-selectable Boolean    1       0}
-            {-drawcross  Enum       auto    0 {auto allways never}}
+            {-drawcross  Enum       auto    0 {auto always never allways}}
 	    {-padx       Int        -1      0 "%d >= -1"}
 	    {-deltax     Int        -1      0 "%d >= -1"}
 	    {-anchor     String     "w"     0 ""}
@@ -221,7 +222,6 @@ proc Tree::create { path args } {
     Tree::bindText  $path <Control-Button-1> [list $selectcmd $path toggle]
     Tree::bindImage $path <Control-Button-1> [list $selectcmd $path toggle]
 
-
     # Add sentinal bindings for double-clicking on items, to handle the 
     # gnarly Tk bug wherein:
     # ButtonClick
@@ -428,6 +428,12 @@ proc Tree::itemcget { path node option } {
     return [Widget::cget $path.$node $option]
 }
 
+# ----------------------------------------------------------------------------
+# Command Tree::bindArea
+# ----------------------------------------------------------------------------
+proc Tree::bindArea { path event script } {
+    bind $path.c $event $script
+}
 
 # ----------------------------------------------------------------------------
 #  Command Tree::bindText
@@ -468,6 +474,7 @@ proc Tree::delete { path args } {
     variable $path
     upvar 0  $path data
 
+    set sel 0
     foreach lnodes $args {
 	foreach node $lnodes {
             set node [_node_name $path $node]
@@ -475,10 +482,14 @@ proc Tree::delete { path args } {
 		set parent [lindex $data($node) 0]
 		set idx	   [lsearch -exact $data($parent) $node]
 		set data($parent) [lreplace $data($parent) $idx $idx]
-		_subdelete $path [list $node]
+		incr sel [_subdelete $path [list $node]]
 	    }
 	}
     }
+    if {$sel} {
+	# if selection changed, call the selectcommand
+	__call_selectcmd $path
+    }
 
     _redraw_idle $path 3
 }
@@ -821,7 +832,7 @@ proc Tree::find {path findInfo {confine ""}} {
     }
 
     if {$found} {
-        if {[string equal $confine "confine"]} {
+        if {![string equal $confine ""]} {
             # test if x stand inside node bbox
 	    set padx [_get_node_padx $path $node]
             set xi [expr {[lindex [$path.c coords n:$node] 0] - $padx}]
@@ -839,7 +850,7 @@ proc Tree::find {path findInfo {confine ""}} {
 
 # ----------------------------------------------------------------------------
 #  Command Tree::line
-#     Returns the line where is drawn a node.
+#     Returns the line where a node was drawn.
 # ----------------------------------------------------------------------------
 proc Tree::line {path node} {
     set node [_node_name $path $node]
@@ -1048,7 +1059,7 @@ proc Tree::edit { path node text {verifycmd ""} {clickres 0} {select 1}} {
         set ok 0
         while { !$ok } {
             tkwait variable Tree::_edit(wait)
-            if { !$_edit(wait) || $verifycmd == "" ||
+            if { !$_edit(wait) || [llength $verifycmd]==0 ||
                  [uplevel \#0 $verifycmd [list $_edit(text)]] } {
                 set ok 1
             }
@@ -1075,7 +1086,7 @@ proc Tree::edit { path node text {verifycmd ""} {clickres 0} {select 1}} {
 #  Command Tree::xview
 # ----------------------------------------------------------------------------
 proc Tree::xview { path args } {
-    return [eval [list $path.c xview] $args]
+    return [eval [linsert $args 0 $path.c xview]]
 }
 
 
@@ -1083,7 +1094,7 @@ proc Tree::xview { path args } {
 #  Command Tree::yview
 # ----------------------------------------------------------------------------
 proc Tree::yview { path args } {
-    return [eval [list $path.c yview] $args]
+    return [eval [linsert $args 0 $path.c yview]]
 }
 
 
@@ -1141,7 +1152,7 @@ proc Tree::_recexpand { path node expand recursive cmd } {
 
     if { [Widget::getoption $path.$node -open] != $expand } {
         Widget::setoption $path.$node -open $expand
-        if { $cmd != "" } {
+        if {[llength $cmd]} {
             uplevel \#0 $cmd [list $node]
         }
     }
@@ -1162,6 +1173,7 @@ proc Tree::_subdelete { path lnodes } {
     upvar 0  $path data
 
     set sel $data(selnodes)
+    set selchanged 0
 
     while { [llength $lnodes] } {
         set lsubnodes [list]
@@ -1173,6 +1185,7 @@ proc Tree::_subdelete { path lnodes } {
 	    set idx [lsearch -exact $sel $node]
 	    if { $idx >= 0 } {
 		set sel [lreplace $sel $idx $idx]
+		incr selchanged
 	    }
             if { [set win [Widget::getoption $path.$node -window]] != "" } {
                 destroy $win
@@ -1183,6 +1196,8 @@ proc Tree::_subdelete { path lnodes } {
     }
 
     set data(selnodes) $sel
+    # return number of sel items changes
+    return $selchanged
 }
 
 
@@ -1232,12 +1247,12 @@ proc Tree::_cross_event { path } {
     set node [Tree::_get_node_name $path current 1]
     if { [Widget::getoption $path.$node -open] } {
         Tree::itemconfigure $path $node -open 0
-        if { [set cmd [Widget::getoption $path -closecmd]] != "" } {
+        if {[llength [set cmd [Widget::getoption $path -closecmd]]]} {
             uplevel \#0 $cmd [list $node]
         }
     } else {
         Tree::itemconfigure $path $node -open 1
-        if { [set cmd [Widget::getoption $path -opencmd]] != "" } {
+        if {[llength [set cmd [Widget::getoption $path -opencmd]]]} {
             uplevel \#0 $cmd [list $node]
         }
     }
@@ -1285,7 +1300,6 @@ proc Tree::_draw_cross { path node open x y } {
 #  Command Tree::_draw_node
 # ----------------------------------------------------------------------------
 proc Tree::_draw_node { path node x0 y0 deltax deltay padx showlines } {
-    global   env
     variable $path
     upvar 0  $path data
 
@@ -1312,7 +1326,8 @@ proc Tree::_draw_node { path node x0 y0 deltax deltay padx showlines } {
                     [expr {$x0+$deltax}] $y0 $deltax $deltay $padx $showlines]
     }
 
-    if {![string equal $dc "never"] && ($len || [string equal $dc "allways"])} {
+    if {![string equal $dc "never"]
+	&& ($len || [string equal $dc "always"] || [string equal $dc "allways"])} {
         _draw_cross $path $node $exp $x0 $y0
     }
 
@@ -1350,6 +1365,11 @@ proc Tree::_draw_subnodes { path nodes x0 y0 deltax deltay padx showlines } {
         set y1 [_draw_node $path $node $x0 [expr {$y1+$deltay}] $deltax $deltay $padx $showlines]
     }
     if { $showlines && [llength $nodes] } {
+	if {$y0 < 0} {
+	    # Adjust the drawing of the line to the first root node
+	    # to start at the vertical point (not go up).
+	    incr y0 $deltay
+	}
         set id [$path.c create line $x0 $y0 $x0 [expr {$yp+$deltay}] \
                     -fill    [Widget::getoption $path -linesfill]   \
                     -stipple [Widget::getoption $path -linestipple] \
@@ -1365,73 +1385,70 @@ proc Tree::_draw_subnodes { path nodes x0 y0 deltax deltay padx showlines } {
 #  Command Tree::_update_nodes
 # ----------------------------------------------------------------------------
 proc Tree::_update_nodes { path } {
-    global   env
     variable $path
     upvar 0  $path data
 
-    set deltax [Widget::getoption $path -deltax]
-    set padx   [Widget::getoption $path -padx]
     foreach {node flag} $data(upd,nodes) {
-        set idn [$path.c find withtag "n:$node"]
-        if { $idn == "" } {
-            continue
-        }
+	set idn [$path.c find withtag "n:$node"]
+	if { $idn == "" } {
+	    continue
+	}
 	set padx   [_get_node_padx $path $node]
 	set deltax [_get_node_deltax $path $node]
-        set c  [$path.c coords $idn]
-        set x0 [expr {[lindex $c 0]-$padx}]
-        set y0 [lindex $c 1]
-        if { $flag & 48 } {
-            # -window or -image modified
-            set win  [Widget::getoption $path.$node -window]
-            set img  [Widget::getoption $path.$node -image]
-            set idi  [$path.c find withtag i:$node]
-            set type [lindex [$path.c gettags $idi] 1]
-            if { [string length $win] } {
-                if { [string equal $type "win"] } {
-                    $path.c itemconfigure $idi -window $win
-                } else {
-                    $path.c delete $idi
-                    $path.c create window $x0 $y0 -window $win -anchor w \
-			    -tags [Tree::_get_node_tags $path $node \
-			    	[list win i:$node]]
-                }
-            } elseif { [string length $img] } {
-                if { [string equal $type "img"] } {
-                    $path.c itemconfigure $idi -image $img
-                } else {
-                    $path.c delete $idi
-                    $path.c create image $x0 $y0 -image $img -anchor w \
-			    -tags [Tree::_get_node_tags $path $node \
-			    	[list img i:$node]]
-                }
-            } else {
-                $path.c delete $idi
-            }
-        }
+	set c  [$path.c coords $idn]
+	set x1 [expr {[lindex $c 0]-$padx}]
+	set x0 [expr {$x1-$deltax-5}]
+	set y0 [lindex $c 1]
+	if { $flag & 48 } {
+	    # -window or -image modified
+	    set win  [Widget::getoption $path.$node -window]
+	    set img  [Widget::getoption $path.$node -image]
+	    set anc  [Widget::cget $path.$node -anchor]
+	    set idi  [$path.c find withtag i:$node]
+	    set type [lindex [$path.c gettags $idi] 1]
+	    if { [string length $win] } {
+		if { [string equal $type "win"] } {
+		    $path.c itemconfigure $idi -window $win
+		} else {
+		    $path.c delete $idi
+		    $path.c create window $x1 $y0 -window $win -anchor $anc \
+			-tags [_get_node_tags $path $node [list win i:$node]]
+		}
+	    } elseif { [string length $img] } {
+		if { [string equal $type "img"] } {
+		    $path.c itemconfigure $idi -image $img
+		} else {
+		    $path.c delete $idi
+		    $path.c create image $x1 $y0 -image $img -anchor $anc \
+			-tags [_get_node_tags $path $node [list img i:$node]]
+		}
+	    } else {
+		$path.c delete $idi
+	    }
+	}
 
-        if { $flag & 8 } {
-            # -drawcross modified
-            set len [expr {[llength $data($node)] > 1}]
-            set dc  [Widget::getoption $path.$node -drawcross]
-            set exp [Widget::getoption $path.$node -open]
+	if { $flag & 8 } {
+	    # -drawcross modified
+	    set len [expr {[llength $data($node)] > 1}]
+	    set dc  [Widget::getoption $path.$node -drawcross]
+	    set exp [Widget::getoption $path.$node -open]
 
-            if {![string equal $dc "never"]
-                && ($len || [string equal $dc "allways"])} {
-                _draw_cross $path $node $exp $x0 $y0
-            } else {
+	    if {![string equal $dc "never"]
+		&& ($len || [string equal $dc "always"] || [string equal $dc "allways"])} {
+		_draw_cross $path $node $exp $x0 $y0
+	    } else {
 		set idc [$path.c find withtag c:$node]
-                $path.c delete $idc
-            }
-        }
+		$path.c delete $idc
+	    }
+	}
 
-        if { $flag & 7 } {
-            # -font, -text or -fill modified
-            $path.c itemconfigure $idn \
-                -text [Widget::getoption $path.$node -text] \
-                -fill [Widget::getoption $path.$node -fill] \
-                -font [Widget::getoption $path.$node -font]
-        }
+	if { $flag & 7 } {
+	    # -font, -text or -fill modified
+	    $path.c itemconfigure $idn \
+		-text [Widget::getoption $path.$node -text] \
+		-fill [Widget::getoption $path.$node -fill] \
+		-font [Widget::getoption $path.$node -font]
+	}
     }
 }
 
@@ -1526,7 +1543,7 @@ proc Tree::_redraw_idle { path level } {
     upvar 0  $path data
 
     if { [Widget::getoption $path -redraw] && $data(upd,afterid) == "" } {
-        set data(upd,afterid) [after idle Tree::_redraw_tree $path]
+        set data(upd,afterid) [after idle [list Tree::_redraw_tree $path]]
     }
     if { $level > $data(upd,level) } {
         set data(upd,level) $level
@@ -1546,7 +1563,7 @@ proc Tree::_init_drag_cmd { path X Y top } {
          [string equal $item "img"]  ||
          [string equal $item "win"] } {
         set node [Tree::_get_node_name $path current 2]
-        if { [set cmd [Widget::getoption $path -draginitcmd]] != "" } {
+        if {[llength [set cmd [Widget::getoption $path -draginitcmd]]]} {
             return [uplevel \#0 $cmd [list $path $node $top]]
         }
         if { [set type [Widget::getoption $path -dragtype]] == "" } {
@@ -1575,10 +1592,10 @@ proc Tree::_drop_cmd { path source X Y op type dnddata } {
         set data(dnd,afterid) ""
     }
     set data(dnd,scroll) ""
-    if { [llength $data(dnd,node)] } {
-        if { [set cmd [Widget::getoption $path -dropcmd]] != "" } {
-            return [uplevel \#0 $cmd [list $path $source $data(dnd,node) $op $type $dnddata]]
-        }
+    if {[llength $data(dnd,node)]
+	&& [llength [set cmd [Widget::getoption $path -dropcmd]]]} {
+	return [uplevel \#0 $cmd \
+		    [list $path $source $data(dnd,node) $op $type $dnddata]]
     }
     return 0
 }
@@ -1724,7 +1741,7 @@ proc Tree::_over_cmd { path source event X Y op type dnddata } {
             }
         }
 
-    if { $vmode && [set cmd [Widget::getoption $path -dropovercmd]] != "" } {
+    if {$vmode && [llength [set cmd [Widget::getoption $path -dropovercmd]]]} {
         # user-defined dropover command
         set res     [uplevel \#0 $cmd [list $path $source $target $op $type $dnddata]]
         set code    [lindex $res 0]
@@ -1823,7 +1840,7 @@ proc Tree::_auto_scroll { path x y } {
 
     set data(dnd,scroll) $scroll
     if { [string length $scroll] && ![string length $data(dnd,afterid)] } {
-        set data(dnd,afterid) [after 200 Tree::_scroll $path $scroll]
+        set data(dnd,afterid) [after 200 [list Tree::_scroll $path $scroll]]
     }
     return $data(dnd,afterid)
 }
@@ -1839,7 +1856,7 @@ proc Tree::_scroll { path cmd dir } {
     if { ($dir == -1 && [lindex [$path.c $cmd] 0] > 0) ||
          ($dir == 1  && [lindex [$path.c $cmd] 1] < 1) } {
         $path.c $cmd scroll $dir units
-        set data(dnd,afterid) [after 100 Tree::_scroll $path $cmd $dir]
+        set data(dnd,afterid) [after 100 [list Tree::_scroll $path $cmd $dir]]
     } else {
         set data(dnd,afterid) ""
         DropSite::setcursor dot
@@ -1863,6 +1880,11 @@ proc Tree::_scroll { path cmd dir } {
 #	None.
 
 proc Tree::_keynav {which win} {
+    # check for an empty tree
+    if {[$win nodes root] eq ""} {
+        return
+    }
+
     # Keyboard navigation is riddled with special cases.  In order to avoid
     # the complex logic, we will instead make a list of all the visible,
     # selectable nodes, then do a simple next or previous operation.
@@ -1936,7 +1958,7 @@ proc Tree::_keynav {which win} {
                 }
             } else {
                 $win itemconfigure $node -open 1
-                if { [set cmd [Widget::getoption $win -opencmd]] != "" } {
+                if {[llength [set cmd [Widget::getoption $win -opencmd]]]} {
                     uplevel \#0 $cmd [list $node]
                 }
                 return
@@ -1951,7 +1973,7 @@ proc Tree::_keynav {which win} {
 	    set open [$win itemcget $node -open]
 	    if { $open } {
 		$win itemconfigure $node -open 0
-                if { [set cmd [Widget::getoption $win -closecmd]] != "" } {
+                if {[llength [set cmd [Widget::getoption $win -closecmd]]]} {
                     uplevel \#0 $cmd [list $node]
                 }
 		return
@@ -1988,13 +2010,13 @@ proc Tree::_keynav {which win} {
 		if {$open} {
 		    # Node was open, is now closed. Call the close-cmd
 
-		    if { [set cmd [Widget::getoption $win -closecmd]] != "" } {
+		    if {[llength [set cmd [Widget::getoption $win -closecmd]]]} {
 			uplevel \#0 $cmd [list $node]
 		    }
 		} else {
 		    # Node was closed, is now open. Call the open-cmd
 
-		    if { [set cmd [Widget::getoption $win -opencmd]] != "" } {
+		    if {[llength [set cmd [Widget::getoption $win -opencmd]]]} {
 			uplevel \#0 $cmd [list $node]
 		    }
                 }
@@ -2021,7 +2043,7 @@ proc Tree::_keynav {which win} {
 proc Tree::_get_current_node {win} {
     if {[info exists selectTree::selectCursor($win)]} {
 	set result $selectTree::selectCursor($win)
-    } elseif {[set selList [$win selection get]] != {}} {
+    } elseif {[llength [set selList [$win selection get]]]} {
 	set result [lindex $selList 0]
     } else {
 	set result ""
@@ -2116,7 +2138,7 @@ proc Tree::_get_node_deltax {path node} {
 # Results:
 #	list		The list of tags to apply to the canvas item
 proc Tree::_get_node_tags {path node {tags ""}} {
-    eval [list lappend list TreeItemSentinal] $tags
+    eval [linsert $tags 0 lappend list TreeItemSentinal]
     if {[Widget::getoption $path.$node -helptext] == ""} { return $list }
 
     switch -- [Widget::getoption $path.$node -helptype] {
@@ -2146,7 +2168,7 @@ proc Tree::_set_help { path node } {
 
     set item $path.$node
     set opts [list -helptype -helptext -helpvar]
-    foreach {cty ctx cv} [eval [list Widget::hasChangedX $item] $opts] break
+    foreach {cty ctx cv} [eval [linsert $opts 0 Widget::hasChangedX $item]] break
     set text [Widget::getoption $item -helptext]
 
     ## If we've never set help for this item before, and text is not blank,
@@ -2172,7 +2194,7 @@ proc Tree::_set_help { path node } {
 }
 
 proc Tree::_mouse_select { path cmd args } {
-    eval selection [list $path] [list $cmd] $args
+    eval [linsert $args 0 selection $path $cmd]
     switch -- $cmd {
         "add" - "clear" - "remove" - "set" - "toggle" {
             event generate $path <<TreeSelect>>
